import 'package:flutter/material.dart';

import 'g_palette.dart';
import 'g_palette_data.dart';
import 'tab_bar_indicator.dart';

class GThemeData {
  final String fontFamily;
  final double buttonBorderRadius;
  final double inputBorderRadius;
  final bool appBarCenterTitle;
  final double appBarElevation;

  GThemeData({
    this.fontFamily = "Poppins",
    this.buttonBorderRadius = 4,
    this.inputBorderRadius = 4,
    this.appBarCenterTitle = false,
    this.appBarElevation = 4,
  });

  ThemeData generate(BuildContext context) {
    final palette = GPalette.paletteOf(context);
    return _generate(palette: palette.lightPalette);
  }

  ThemeData generateDark(BuildContext context) {
    final palette = GPalette.paletteOf(context);
    return _generate(palette: palette.darkPalette);
  }

  ThemeData _generate({required GPaletteData palette}) {
    final textTheme = _textTheme(fontFamily).apply(
      displayColor: palette.onBackground,
      bodyColor: palette.onBackground,
      decorationColor: palette.onBackground,
    );

    return ThemeData(
      fontFamily: fontFamily,
      colorScheme: palette.toColorScheme(),
      textTheme: textTheme,
      primaryColor: palette.primary,
      indicatorColor: palette.primary,
      highlightColor: Colors.black.withOpacity(.25),
      toggleableActiveColor: palette.primary,
      scaffoldBackgroundColor: palette.background,
      popupMenuTheme: PopupMenuThemeData(color: palette.surface),
      dialogBackgroundColor: palette.surface,
      dividerColor: palette.gray,
      tooltipTheme: _tooltipThemeData(palette),
      appBarTheme: _appBarTheme(palette, textTheme),
      drawerTheme: DrawerThemeData(backgroundColor: palette.surface),
      tabBarTheme: _tabBarTheme(palette),
      bottomAppBarTheme: BottomAppBarTheme(color: palette.surface),
      bottomNavigationBarTheme: _bottomNavBarThemeData(palette, textTheme),
      bottomSheetTheme: BottomSheetThemeData(backgroundColor: palette.surface),
      elevatedButtonTheme: _elevatedbuttonStyle(palette),
      textButtonTheme: TextButtonThemeData(style: _buttonStyle(palette)),
      outlinedButtonTheme:
          OutlinedButtonThemeData(style: _buttonStyle(palette)),
      floatingActionButtonTheme: _fabThemeData(palette),
    );
  }

  TextTheme _textTheme(String fontFamily) {
    return TextTheme(
      headline1: TextStyle(
        fontFamily: fontFamily,
        fontSize: 96,
        fontWeight: FontWeight.w300,
        letterSpacing: -1.5,
      ),
      headline2: TextStyle(
        fontFamily: fontFamily,
        fontSize: 60,
        fontWeight: FontWeight.w300,
        letterSpacing: -0.5,
      ),
      headline3: TextStyle(
        fontFamily: fontFamily,
        fontSize: 48,
        fontWeight: FontWeight.w400,
      ),
      headline4: TextStyle(
        fontFamily: fontFamily,
        fontSize: 34,
        fontWeight: FontWeight.w400,
        letterSpacing: 0.25,
      ),
      headline5: TextStyle(
        fontFamily: fontFamily,
        fontSize: 24,
        fontWeight: FontWeight.w400,
      ),
      headline6: TextStyle(
        fontFamily: fontFamily,
        fontSize: 20,
        fontWeight: FontWeight.w500,
        letterSpacing: 0.15,
      ),
      subtitle1: TextStyle(
        fontFamily: fontFamily,
        fontSize: 16,
        fontWeight: FontWeight.w400,
        letterSpacing: 0.15,
      ),
      subtitle2: TextStyle(
        fontFamily: fontFamily,
        fontSize: 14,
        fontWeight: FontWeight.w500,
        letterSpacing: 0.1,
      ),
      bodyText1: TextStyle(
        fontFamily: fontFamily,
        fontSize: 16,
        fontWeight: FontWeight.w400,
        letterSpacing: 0.5,
      ),
      bodyText2: TextStyle(
        fontFamily: fontFamily,
        fontSize: 14,
        fontWeight: FontWeight.w400,
        letterSpacing: 0.25,
      ),
      button: TextStyle(
        fontFamily: fontFamily,
        fontSize: 14,
        fontWeight: FontWeight.w500,
        letterSpacing: 1.25,
      ),
      caption: TextStyle(
        fontFamily: fontFamily,
        fontSize: 12,
        fontWeight: FontWeight.w400,
        letterSpacing: 0.4,
      ),
      overline: TextStyle(
        fontFamily: fontFamily,
        fontSize: 10,
        fontWeight: FontWeight.w400,
        letterSpacing: 1.5,
      ),
    );
  }

  TooltipThemeData _tooltipThemeData(GPaletteData palette) {
    return TooltipThemeData(
      decoration: BoxDecoration(
        color: palette.gray.withOpacity(.75),
        borderRadius: BorderRadius.circular(4),
      ),
    );
  }

  AppBarTheme _appBarTheme(GPaletteData palette, TextTheme textTheme) {
    return AppBarTheme(
      foregroundColor: palette.onSurface,
      backgroundColor: palette.surface,
      centerTitle: appBarCenterTitle,
      elevation: appBarElevation,
      titleTextStyle: textTheme.headline6?.copyWith(
        fontWeight: FontWeight.normal,
        color: palette.onSurface,
      ),
      iconTheme: IconThemeData(color: palette.onSurface),
      shadowColor: palette.brightness == Brightness.light
          ? Colors.white70
          : Colors.black45,
    );
  }

  TabBarTheme _tabBarTheme(GPaletteData palette) {
    return TabBarTheme(
      labelColor: palette.primary,
      unselectedLabelColor: palette.gray.withOpacity(.75),
      indicator: TabBarIndicator(palette.primary),
      indicatorSize: TabBarIndicatorSize.label,
    );
  }

  BottomNavigationBarThemeData _bottomNavBarThemeData(
    GPaletteData palette,
    TextTheme textTheme,
  ) {
    return BottomNavigationBarThemeData(
      type: BottomNavigationBarType.fixed,
      backgroundColor: palette.surface,
      selectedItemColor: palette.onSurface,
      unselectedItemColor: palette.onSurface,
      selectedLabelStyle: textTheme.caption,
      unselectedLabelStyle: textTheme.caption,
    );
  }

  ElevatedButtonThemeData _elevatedbuttonStyle(GPaletteData palette) {
    return ElevatedButtonThemeData(
      style: ButtonStyle(
        backgroundColor: MaterialStateColor.resolveWith(
          (states) => palette.primary,
        ),
        overlayColor: MaterialStateColor.resolveWith(
          (states) => Colors.black.withOpacity(.25),
        ),
        elevation: MaterialStateProperty.resolveWith(
          (states) {
            const pressedState = MaterialState.pressed;
            const hoveredState = MaterialState.hovered;
            final isHoveredOrPressed = states
                .where((e) => e == pressedState || e == hoveredState)
                .isNotEmpty;

            if (isHoveredOrPressed) {
              return 3;
            }

            return 0;
          },
        ),
        shape: MaterialStateProperty.resolveWith(
          (states) => RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(buttonBorderRadius),
          ),
        ),
        tapTargetSize: MaterialTapTargetSize.shrinkWrap,
      ),
    );
  }

  ButtonStyle _buttonStyle(GPaletteData palette) {
    return ButtonStyle(
      visualDensity: VisualDensity.standard,
      tapTargetSize: MaterialTapTargetSize.shrinkWrap,
      shape: MaterialStateProperty.resolveWith(
        (states) => RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(buttonBorderRadius),
        ),
      ),
    );
  }

  FloatingActionButtonThemeData _fabThemeData(
    GPaletteData palette,
  ) {
    return FloatingActionButtonThemeData(
      backgroundColor: palette.primary,
      foregroundColor: palette.onPrimary,
      splashColor: Colors.black.withOpacity(.25),
    );
  }
}
